<?php
// Интерфейс для драйверов ботов

namespace BotKit\Drivers;

use BotKit\Common\User;
use BotKit\Common\Chat;
use BotKit\Common\Message;
use BotKit\Common\EventData;
use BotKit\Events\Event;
use BotKit\Events\PlainMessageEvent;

interface Driver {

    // Возвращает true, если драйвер считает, что именно ему необходимо обработать этот запрос
    public function forThis() : bool;

    // Возвращает событие на основании данных запроса
    public function getEvent() : Event;

    // Отвечает на текстовое сообщение
    // Если empathise=true, сообщение должно быть явным ответом
    // $e - событие, на которое создаётся ответ
    // $msg - сообщение ответа
    public function reply(PlainMessageEvent $e, Message $msg, bool $empathise = true);

    // Отсылает сообщение пользователю
    public function sendMessage(User $u, Message $msg);

    // Отправляет сообщение в чат
    public function sendToChat(Chat $chat, Message $msg);

    // Возвращает ник пользователя, например @aquadim
    public function getUserNick(User $u) : string;
}